/*
 *
 *
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>

#include "utils/dlist.h"

#include "tdm_common.h"
#include "tdm_object.h"
#include "tdm_log.h"

#include "tdm_channel.h"
#include "tdm_span.h"

struct tdm_channel *tdm_channel_new(const char *name)
{
	struct tdm_channel *chan = NULL;

	chan = (struct tdm_channel *)tdm_object_new(name, sizeof(*chan));
	if (!chan)
		return NULL;

	tdm_object_set_type(chan, TDM_OBJECT_CHANNEL);

	dlist_init(&chan->span_channel_list);
	dlist_init(&chan->port_channel_list);

	return chan;
}

struct tdm_module *tdm_channel_get_module(struct tdm_channel *chan)
{
	if (!chan || !tdm_object_is_channel(chan))
		return NULL;

	return (struct tdm_module *)tdm_object_owner(chan);
}

void tdm_channel_set_owner(struct tdm_channel *chan, struct tdm_module *owner)
{
	if (!chan || !tdm_object_is_channel(chan))
		return;

	if (!owner || !tdm_object_is_module(owner))
		return;

	tdm_object_set_owner(chan, owner);
}

struct tdm_channel *tdm_object_to_channel(struct tdm_object *obj)
{
	if (!obj || !tdm_object_is_channel(obj))
		return NULL;

	return (struct tdm_channel *)obj;
}

struct tdm_span *tdm_channel_get_span(struct tdm_channel *chan)
{
	struct tdm_span   *span = NULL;
	struct dlist_head *head;

	if (!chan || !tdm_object_is_channel(chan))
		return NULL;
#if 0
	head = dlist_get_head(&chan->span_channel_list);
	if (!head) {
		tdm_error("%s: orphaned or unassigned channel\n", tdm_object_name(chan));
		return NULL;
	}

	span = dlist_entry(head, struct tdm_span, channel_list);
	if (!tdm_object_is_span(span)) {
		tdm_error("%p: object is not a span object\n", span);
		return NULL;
	}
#else
	span = chan->span;
	if (!tdm_object_is_span(span)) {
		tdm_error("%p: object is not a span object\n", span);
		return NULL;
	}
#endif
	return span;
}

/* bad naming galore */
int tdm_channel_has_span(struct tdm_channel *chan)
{
	if (!chan || !tdm_object_is_channel(chan))
		return 0;
#if 0
	return !!(dlist_get_head(&chan->span_channel_list));
#else
	return !!(chan->span);
#endif
}
