/*
 * libtdm utility functions
 *
 */
#include <stdio.h>
#include <stdarg.h>
#include <limits.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

//#include "tdm_private.h"
#include "tdm_common.h"
#include "tdm_log.h"

#define TDM_THREAD_NAME_MAX	50


#if !defined(HAVE_GETTID)
#include <sys/syscall.h>
static int gettid(void)
{
	return syscall(SYS_gettid);
}
#endif

/**
 * Set name of current thread
 */
int tdm_thread_set_name(const char *name)
{
#if defined(HAVE_PTHREAD_SETNAME_NP)
	return pthread_setname_np(pthread_self(), name);
#else
	char path[PATH_MAX] = { 0 };
	int ret = TDM_STATUS_GENERR;
	int fd  = -1;

	snprintf(path, sizeof(path), "/proc/self/task/%d/comm", gettid());
	fd = open(path, O_WRONLY);
	if (fd < 0) {
		tdm_debug("Failed to open \"%s\" to set thread name\n", path);
		return ret;
	}

	if (write(fd, name, strlen(name) + 1) < 0) {
		tdm_debug("Failed to set thread name \"%s\"\n", name);
		goto out;
	}

	ret = TDM_STATUS_SUCCESS;
out:
	close(fd);
	return ret;
#endif
}

/**
 * Set (formatted) name of current thread
 */
int tdm_thread_set_name_sprintf(const char *fmt, ...)
{
	char name[TDM_THREAD_NAME_MAX] = { 0 };
	va_list ap;

	va_start(ap, fmt);
	vsnprintf(name, sizeof(name), fmt, ap);
	va_end(ap);

	return tdm_thread_set_name(name);
}
