
#include "common.h"
#include "hlist.h"


void hlist_init_head( struct hlist_head *h )
{
	h->first = NULL;
}

void hlist_init_elem( struct hlist_elem *e )
{
	e->next = NULL;
	e->prev = NULL;
}

struct hlist_elem * hlist_first( struct hlist_elem *elem )
{
	struct hlist_elem *p = elem;

	for (; p && p->prev; p = p->prev);

	return p;
}

struct hlist_elem * hlist_first_elem( struct hlist_head *list )
{
	return list->first;
}

struct hlist_elem * hlist_last( struct hlist_elem *elem )
{
	struct hlist_elem *p = elem;

	for (; p && p->next; p = p->next);

	return p;
}

struct hlist_elem * hlist_last_elem( struct hlist_head *list )
{
	struct hlist_elem *p = list->first;

	for (;p && p->next; p = p->next);

	return p;
}

struct hlist_elem * hlist_next( struct hlist_elem *elem )
{
	return elem->next;
}

struct hlist_elem * hlist_prev( struct hlist_elem *elem )
{
	return elem->prev;
}

int hlist_is_first( struct hlist_elem *elem )
{
	return (elem->prev == NULL);
}


int hlist_is_last( struct hlist_elem *elem )
{
	return (elem->next == NULL);
}


void hlist_insert_after( struct hlist_head *list, struct hlist_elem *elem, struct hlist_elem *enew )
{
	if (elem->next)
		elem->next->prev = enew;

	enew->next = elem->next;
	enew->prev = elem;
	elem->next = enew;
}


void hlist_insert_before( struct hlist_head *list, struct hlist_elem *elem, struct hlist_elem *enew )
{
	if (elem->prev)
		elem->prev->next = enew;
	else
		list->first = enew;
	enew->prev = elem->prev;
	enew->next = elem;
	elem->prev = enew;
}

void hlist_insert_tail( struct hlist_head *list, struct hlist_elem *enew )
{
	if (hlist_is_empty(list)) {
		hlist_init_elem(enew);
		list->first = enew;
	} else
		hlist_insert_after( list, hlist_last_elem(list), enew );
}

void hlist_remove( struct hlist_head *list, struct hlist_elem *elem )
{
	/*
	 * Cannot remove list head
	 */
	if (hlist_is_first(elem))
		list->first = elem->next;

	if (!hlist_is_last(elem)) {
		elem->prev->next = elem->next;
		elem->next->prev = elem->prev;
	} else
		elem->prev->next = NULL;
}

void hlist_replace( struct hlist_head *list, struct hlist_elem *eold, struct hlist_elem *enew )
{
	if (hlist_is_first(eold))
		list->first = enew;

	enew->next = eold->next;

	if (!hlist_is_last(eold))
		enew->next->prev = enew;

	enew->prev = eold->prev;
	enew->prev->next = enew;
}
