
#include "common.h"
#include "slist.h"


void slist_init_head( struct slist_head *h )
{
	h->head = h;
	h->next = h;
}

struct slist_head * slist_first( struct slist_head *list )
{
	if (list->head && list->head->next != list->head)
		return list->head->next;

	return NULL;
}

struct slist_head * slist_last( struct slist_head *list )
{
	struct slist_head *cur = list->head;

	for (; cur->next; cur = cur->next);

	return cur;
}

struct slist_head * slist_next( struct slist_head *elem )
{
	return elem->next;
}

int slist_is_head( struct slist_head *elem )
{
	return (elem->head == elem);
}


int slist_is_first( struct slist_head *elem )
{
	return (elem->head->next == elem);
}


int slist_is_last( struct slist_head *elem )
{
	return (elem->next == elem->head);
}


void slist_insert_after( struct slist_head *elem, struct slist_head *enew )
{
	enew->head = elem->head;
	enew->next = elem->next;
	elem->next = enew;
}

void slist_insert_before( struct slist_head *elem, struct slist_head *enew )
{
	struct slist_head *prev;

	/* find our parent (= previous) node */
	for (prev = elem->head->next; prev->next != elem && prev->next != elem->head; prev = prev->next);

	enew->head = prev->head;
	enew->next = prev->next;
	prev->next = enew;
}


void slist_insert_tail( struct slist_head *list, struct slist_head *enew )
{
	slist_insert_after( slist_last(list), enew );
}


void slist_remove( struct slist_head *elem )
{
	struct slist_head *prev;

	/* can not remove list head */
	if (slist_is_head(elem))
		return;

	/* find our parent (= previous) node */
	for (prev = elem->head->next; prev->next != elem; prev = prev->next);

	prev->next = elem->next;
}


void slist_replace( struct slist_head *elem, struct slist_head *enew )
{
	struct slist_head *prev;

	if (slist_is_head(elem))	/* can not replace head */
		return;

	/* find our parent (= previous) node */
	for (prev = elem->head; prev->next != elem; prev = prev->next);

	enew->next = elem->next;
	enew->head = elem->head;
	prev->next = enew;
	elem->next = NULL;
	elem->head = NULL;
}
