#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include "libsangoma.h"


#if 0
#define TDM_LIB_MAXNAME	50

struct tdm_lib_group {
	char name[TDM_LIB_MAXNAME];

	struct list_head span_list;	/* span list */
};

int   tdm_lib_group_set_name(struct tdm_lib_group *group, const char *name);
char *tdm_lib_group_get_name(const struct tdm_lib_group *group);

int   tdm_lib_group_foreach_span(struct tdm_lib_group *group, void *callback, void *data);
int   tdm_lib_group_add_span(struct tdm_lib_group *group, struct tdm_lib_span *span);
int   tdm_lib_group_remove_span(struct tdm_lib_group *group, struct tdm_lib_span *span);

int   tdm_lib_group_add_span(struct tdm_lib_group *group, struct tdm_lib_span *span)
{
	if (!group || !span)
		return -EINVAL;

	/* already in a group? */
	if (!list_is_head(&span->group_span_list))
		return -EBUSY;

	return list_append(&group->span_list, &span->group_span_list);
}

int   tdm_lib_group_remove_span(struct tdm_lib_group *group, struct tdm_lib_span *span)
{
	if (!group || !span)
		return -EINVAL;

	/* not in a group? */
	if (list_is_head(&span->group_span_list))
		return -EINVAL;

	/* owned by somebody else? */
	if (tdm_lib_span_get_group(span) != group)
		return -EACCESS;

	return list_remove(&span->group_span_list);
}



struct tdm_lib_span {
	int id;

	void *private;			/* user private data */

	struct list_head chan_list;		/* channel list */
	struct list_head group_span_list;	/* group's span list */
};

int   tdm_lib_span_foreach_channel(struct tdm_lib_group *group, void *callback, void *data);

int   tdm_lib_span_in_group(const struct tdm_lib_span *span)
{
	return list_is_head(&span->group_span_list);
}

struct tdm_lib_group *tdm_lib_span_get_group(const struct tdm_lib_span *span);
struct tdm_lib_group *tdm_lib_span_get_group(const struct tdm_lib_span *span)
{
	if (!span || list_is_head(&span->group_span_list))
		return NULL;

	return container_of(list_head(&span->group_span_list), struct tdm_lib_group, span_list);
}


/* high-level interface, e.g. (_dial() being a bad example, more like _request_channel()) */
int tdm_lib_span_dial(struct tdm_lib_span *span, void *params)
{
	/* in a group? */
	if (tdm_lib_span_in_group(span))
		return tdm_lib_group_dial(tdm_lib_span_get_group(span), params);

	if (!span->dial)
		return -ENOTSUP;

	return span->dial(span, params);
}


struct tdm_lib_channel {
	int id;

	void *private;		/* user private data */

	struct list_head port_chan_list;	/* port's channel list */
	struct list_head span_chan_list;	/* span's channel list */
};

struct tdm_lib_port *tdm_lib_channel_get_port(const struct tdm_lib_channel *channel);
struct tdm_lib_port *tdm_lib_channel_get_port(const struct tdm_lib_channel *channel)
{
	return container_of(list_head(&channel->port_chan_list), struct tdm_lib_port, port_chan_list);
}

struct tdm_lib_port *tdm_lib_channel_get_span(const struct tdm_lib_channel *channel);
struct tdm_lib_span *tdm_lib_channel_get_span(const struct tdm_lib_channel *channel)
{
	return container_of(list_head(&channel->span_chan_list), struct tdm_lib_span, chan_list);
}

struct tdm_lib_port {
	int id;
	void *private;			/* i/o-driver private data */

	struct list_head port_list;	/* device port list */
	struct list_head port_chan_list;	/* port's list of channels */
};
int   tdm_lib_port_set_private(struct tdm_lib_port *dev, const void *private)	{ if (!port) return -1; port->private = private; }
void* tdm_lib_port_get_private(const struct tdm_lib_port *dev)			{ if (!port) return NULL; return port->private; }
int   tdm_lib_port_set_id(struct tdm_lib_port *port, int id)			{ if (!port) return -1; port->id = id; return 0; }
int   tdm_lib_port_get_id(const struct tdm_lib_port *port)			{ if (!port) return -1; return port->id; }

int   tdm_lib_port_foreach_channel(struct tdm_lib_port *port, void *callback, void *data);

struct tdm_lib_device *tdm_lib_port_get_device(const struct tdm_lib_port *port);
struct tdm_lib_device *tdm_lib_port_get_device(const struct tdm_lib_port *port)
{
	return container_of(list_head(&channel->port_list), struct tdm_lib_device, port_list);
}


struct tdm_lib_device {
	char name[TDM_LIB_MAXNAME];
	void *private;			/* i/o-driver private data */

	struct list_head device_list;	/* i/o-driver device list */
	struct list_head port_list;	/* list of ports belonging to this device */
};
int   tdm_lib_device_set_private(struct tdm_lib_device *dev, const void *private)	{ if (!dev) return -1; dev->private = private; }
void* tdm_lib_device_get_private(const struct tdm_lib_device *dev)			{ if (!dev) return NULL; return dev->private; }
int   tdm_lib_device_set_name(struct tdm_lib_device *dev, const char *name)		{ int len = 0; if (!dev || !name) return -1; len = strlen(name); if (len >= TDM_LIB_MAXNAME || !len) return -1; memcpy(dev->name, name, len); dev->name[len] = '\0'; return 0; }
char* tdm_lib_device_get_name(const struct tdm_lib_device *dev)				{ if (!dev) return NULL; return dev->name; }

int   tdm_lib_device_add_port(struct tdm_lib_device *device, struct tdm_lib_port *port);
int   tdm_lib_device_remove_port(struct tdm_lib_device *device, struct tdm_lib_port *port);

int   tdm_lib_device_foreach_channel(struct tdm_lib_device *device, void *callback, void *data);
int   tdm_lib_device_foreach_port(struct tdm_lib_device *device, void *callback, void *data);


int tdm_lib_device_add_port(struct tdm_lib_device *device, struct tdm_lib_port *port)
{
	return list_append(&device->port_list, &port->port_list);
}

int tdm_lib_device_remove_port(struct tdm_lib_device *device, struct tdm_lib_port *port)
{
	return list_remove(&port->port_list);
}

struct tdm_lib_port *tdm_lib_device_find_port(struct tdm_lib_device *device, const int port_id)
{
	return NULL;
}


// global module data
static struct {
	int ctrl_fd;

	struct list_head device_list;	/* list of (struct tdm_lib_devices) devices (= cards), detected/owned by this driver */
} sng_globals;

// module init/exit function
static int sng_init(void) {}
static int sng_fini(void) {}


struct tdm_lib_module {

} sng_module = {
	.init = sng_init,	/* module init function */
	.fini = sng_fini,	/* module exit function */

	.probe = sng_probe,	/* detect devices */

	/* span(/device/port) control */
	.start = sng_span_start,
	.stop  = sng_span_stop,
};
#endif





int main(void)
{
	int ctrl_fd = -1;

	printf("Using libsangoma %s\n", LIBSANGOMA_VERSION_STR);

	/* open control iface */
	ctrl_fd = sangoma_open_api_ctrl();
	if (ctrl_fd < 0) {
		perror("Failed to open ctrl channel");
		return -1;
	}


	/* cleanup */
	sangoma_close(&ctrl_fd);

	return 0;
}
